"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.meta = undefined;
exports.default = NetlifyCmsModule;

var _path = require("path");

var _lodash = require("lodash");

var _lodash2 = _interopRequireDefault(_lodash);

var _nuxt = require("nuxt");

var _chokidar = require("chokidar");

var _chokidar2 = _interopRequireDefault(_chokidar);

var _pify = require("pify");

var _pify2 = _interopRequireDefault(_pify);

var _webpack = require("webpack");

var _webpack2 = _interopRequireDefault(_webpack);

var _webpackDevMiddleware = require("webpack-dev-middleware");

var _webpackDevMiddleware2 = _interopRequireDefault(_webpackDevMiddleware);

var _serveStatic = require("serve-static");

var _serveStatic2 = _interopRequireDefault(_serveStatic);

var _debug = require("debug");

var _debug2 = _interopRequireDefault(_debug);

var _package = require("../package.json");

var _package2 = _interopRequireDefault(_package);

var _yaml = require("./utils/yaml.js");

var _netlifyConfig = require("./utils/netlify.config.js");

var _webpackConfig = require("./webpack.config.js");

var _webpackConfig2 = _interopRequireDefault(_webpackConfig);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

/* eslint-disable import/no-extraneous-dependencies */
/* covered by nuxt */


var debug = (0, _debug2.default)("nuxt:netlify-cms");

var WEBPACK_CLIENT_COMPILER_NAME = "client";
var WEBPACK_NETLIFY_COMPILER_NAME = "netlify-cms";

// Defaults
var DEFAULTS = {
  adminPath: "admin",
  adminTitle: "Content Manager",
  cmsConfig: {
    media_folder: "static/uploads"
  }
};

function NetlifyCmsModule(moduleOptions) {
  var _this = this;

  var NETLIFY_CONFIG_FILE_NAME = (0, _path.join)(this.options.rootDir, "netlify-cms.yml");

  var getNetlifyCmsConfigFromFile = function getNetlifyCmsConfigFromFile() {
    return (0, _yaml.loadYAMLFile)(NETLIFY_CONFIG_FILE_NAME) || {};
  };

  var transformNetlifyCmsConfigPaths = function transformNetlifyCmsConfigPaths(config) {
    return (0, _netlifyConfig.setConfigPaths)(config, (0, _path.relative)(_this.options.rootDir, _this.options.srcDir));
  };

  var getWebpackNetlifyConfig = function getWebpackNetlifyConfig(builder, adminPath, adminTitle) {
    return _webpackConfig2.default.call(builder, WEBPACK_NETLIFY_COMPILER_NAME, adminPath, adminTitle);
  };

  var getDistDir = function getDistDir(adminPath) {
    return (0, _path.resolve)(_this.options.buildDir, "dist", adminPath);
  };

  var getOptions = function getOptions() {
    var netlifyCmsConfig = getNetlifyCmsConfigFromFile();
    var options = Object.assign({}, DEFAULTS, {
      cmsConfig: Object.assign({}, netlifyCmsConfig)
    }, _this.options.netlifyCms, moduleOptions);
    options.cmsConfig = transformNetlifyCmsConfigPaths(options.cmsConfig);
    return options;
  };

  var options = getOptions();
  var ADMIN_PATH = options.adminPath.replace(/\/?$/, "/");
  var ADMIN_TITLE = options.adminTitle;
  var DIST_DIR = getDistDir(ADMIN_PATH);

  // This will be called once when builder started
  this.nuxt.plugin("build", function () {
    var _ref = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee3(builder) {
      return regeneratorRuntime.wrap(function _callee3$(_context3) {
        while (1) {
          switch (_context3.prev = _context3.next) {
            case 0:
              // This will be run just before webpack compiler starts
              builder.plugin("compile", function (_ref2) {
                var builder = _ref2.builder,
                    compiler = _ref2.compiler;

                var webpackConfig = getWebpackNetlifyConfig(builder, ADMIN_PATH, ADMIN_TITLE);

                webpackConfig.plugins.push({
                  apply: function apply(compiler) {
                    compiler.plugin("emit", function (compilation, cb) {
                      var netlifyConfigYAML = (0, _yaml.toYAML)(getOptions().cmsConfig);
                      compilation.assets["config.yml"] = {
                        source: function source() {
                          return netlifyConfigYAML;
                        },
                        size: function size() {
                          return netlifyConfigYAML.length;
                        }
                      };
                      cb();
                    });
                  }
                });

                var netlifyCompiler = (0, _webpack2.default)(webpackConfig);

                // Only add the compiler in production,
                // in dev watch will be started by dev-middleware
                if (!_this.options.dev) {
                  compiler.compilers.push(netlifyCompiler);
                  compiler[netlifyCompiler.name] = netlifyCompiler;
                }

                // Use shared filesystem and cache
                var clientCompiler = compiler[WEBPACK_CLIENT_COMPILER_NAME];
                netlifyCompiler.outputFileSystem = clientCompiler.outputFileSystem;
                netlifyCompiler.cache = clientCompiler.cache;

                // This will be run just after webpack compiler ends
                netlifyCompiler.plugin("done", function () {
                  var _ref3 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee(stats) {
                    return regeneratorRuntime.wrap(function _callee$(_context) {
                      while (1) {
                        switch (_context.prev = _context.next) {
                          case 0:
                            if (!stats.hasErrors()) {
                              _context.next = 2;
                              break;
                            }

                            return _context.abrupt("return");

                          case 2:
                          case "end":
                            return _context.stop();
                        }
                      }
                    }, _callee, _this);
                  }));

                  return function (_x2) {
                    return _ref3.apply(this, arguments);
                  };
                }());

                // Create webpack dev middleware in development
                if (_this.options.dev) {
                  var netlifyWebpackDevMiddleware = (0, _pify2.default)((0, _webpackDevMiddleware2.default)(netlifyCompiler, {
                    publicPath: "/",
                    stats: builder.webpackStats,
                    noInfo: true,
                    quiet: true,
                    watchOptions: _this.options.watchers.webpack
                  }));

                  // Inject to renderer instance
                  if (builder.nuxt.renderer) {
                    builder.nuxt.renderer.netlifyWebpackDevMiddleware = netlifyWebpackDevMiddleware;
                  }
                }
              });

              // Show a message inside console when the build is ready
              builder.plugin("compiled", _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee2() {
                return regeneratorRuntime.wrap(function _callee2$(_context2) {
                  while (1) {
                    switch (_context2.prev = _context2.next) {
                      case 0:
                        debug("Serving on: " + ADMIN_PATH);

                      case 1:
                      case "end":
                        return _context2.stop();
                    }
                  }
                }, _callee2, _this);
              })));

            case 2:
            case "end":
              return _context3.stop();
          }
        }
      }, _callee3, _this);
    }));

    return function (_x) {
      return _ref.apply(this, arguments);
    };
  }());

  // Serve netlify CMS
  if (this.options.dev) {
    // Insert webpackDevMiddleware to serve netlify CMS in development
    this.addServerMiddleware({
      path: ADMIN_PATH,
      handler: function () {
        var _ref5 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee4(req, res) {
          return regeneratorRuntime.wrap(function _callee4$(_context4) {
            while (1) {
              switch (_context4.prev = _context4.next) {
                case 0:
                  if (!_this.nuxt.renderer.netlifyWebpackDevMiddleware) {
                    _context4.next = 4;
                    break;
                  }

                  debug("requesting url: " + _nuxt.Utils.urlJoin(ADMIN_PATH, req.url));
                  _context4.next = 4;
                  return _this.nuxt.renderer.netlifyWebpackDevMiddleware(req, res);

                case 4:
                case "end":
                  return _context4.stop();
              }
            }
          }, _callee4, _this);
        }));

        function handler(_x3, _x4) {
          return _ref5.apply(this, arguments);
        }

        return handler;
      }()
    });

    // Stop webpack middleware on nuxt.close()
    this.nuxt.plugin("close", _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee5() {
      return regeneratorRuntime.wrap(function _callee5$(_context5) {
        while (1) {
          switch (_context5.prev = _context5.next) {
            case 0:
              _context5.next = 2;
              return _this.nuxt.renderer.netlifyWebpackDevMiddleware.close();

            case 2:
            case "end":
              return _context5.stop();
          }
        }
      }, _callee5, _this);
    })));

    // Start watching config file
    var patterns = [_nuxt.Utils.r(NETLIFY_CONFIG_FILE_NAME)];

    var _options = Object.assign({}, this.options.watchers.chokidar, {
      ignoreInitial: true
    });

    var refreshFiles = _lodash2.default.debounce(function () {
      return _this.nuxt.renderer.netlifyWebpackDevMiddleware.invalidate();
    }, 200);

    // Watch for src Files
    var fileWatcher = _chokidar2.default.watch(patterns, _options).on("add", refreshFiles).on("change", refreshFiles).on("unlink", refreshFiles);

    this.nuxt.renderer.netlifyFileWatcher = fileWatcher;

    // Stop watching on nuxt.close()
    this.nuxt.plugin("close", function () {
      _this.nuxt.renderer.netlifyFileWatcher.close();
    });
  } else {
    // Statically serve netlify CMS (i.e. .nuxt/dist/admin/) files in production
    this.addServerMiddleware({
      path: ADMIN_PATH,
      handler: (0, _serveStatic2.default)(DIST_DIR, {
        maxAge: "1y" // 1 year in production
      })
    });
  }
}

// REQUIRED if publishing as an NPM package
exports.meta = _package2.default;